//
// rrbrowser.cxx
//
// Browser test program for the Fast Light Tool Kit (FLTK).
//  testing the DND behavior under different platforms
//

#include <FL/Fl.H>
#include <FL/Fl_Select_Browser.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Choice.H>
#include <FL/fl_ask.H>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

//  -------- new stuff -------------
char DBuf[512] = "";

class RRBrowser : public Fl_Select_Browser {
public:
  RRBrowser(int X, int Y, int W, int H, const char * L=0);
  ~RRBrowser();
  virtual void draw();
private:
  int handle(int);
};

RRBrowser::RRBrowser(int X, int Y, int W, int H, const char * L) : Fl_Select_Browser(X, Y, W, H, L) { }

RRBrowser:: ~RRBrowser() { }

void RRBrowser::draw() {
  static int n = 0;
  printf("- draw(%2d) -\n",n++);
  if (n > 99) n = 0;
  Fl_Select_Browser::draw();
  return;
}

int RRBrowser::handle(int ev)
{
  int ret = 0;
  int xpos = Fl::event_x() - x();
  int ypos = Fl::event_y() - y();
  void *t = 0;
  switch (ev) {
  case FL_PUSH:
    t = find_item(ypos +y()); // find out which line is affected; i.e. where the mouse is above
    if (t && item_selected(t)) {
      // something was selected; so we can start a dnd operation
      strcpy(DBuf,text(lineno(t)));
	  Fl::copy(DBuf,strlen(DBuf),0);
      Fl::dnd();
      deselect();
      ret = 1;
    }
    break;

  case FL_DRAG:
    // find out which line is affected; i.e. where the mouse is above
    t = find_item(ypos +y());
    if (0 && t) {
      deselect();
      select(lineno(t));  // highlight the drop target
      printf("DRAG: xpos = %d, ypos = %d\n",xpos,ypos);
      redraw();
      ret = 1;
    }
    break;

  case FL_RELEASE:
    printf("REL: xpos = %d, ypos = %d, DBuf=%s\n",xpos,ypos,DBuf);
    ret = 1;
    break;
  case FL_DND_DRAG:
    printf("DND_DRAG: xpos = %d, ypos = %d\n",xpos,ypos);
    // find out which line is affected; i.e. where the mouse is above
    t = find_item(ypos +y());
    if (t) {
      int n = lineno(t);
      printf ("          lineno = %d: deselect(); select(%d); redraw();\n",n,n);
      deselect();
      select(n);  // highlight the drop target
	  Fl::flush();	//		>>>> This is the workaround! Should not be necessary! <<<<<
    }
    ret = 1;
    break;

  case FL_DND_RELEASE:

    printf("DND_REL: xpos = %d, ypos = %d\n",xpos,ypos);
    t = find_item(ypos +y());
    if (t) {
	
      //insert(lineno(t),Fl::event_text());
      insert(lineno(t),DBuf);
      *DBuf = '\0'; //  clear buffer
      ret = 1;
    } else
		add(DBuf);


    break;

  case FL_DND_ENTER:
    printf("DND_ENTER: xpos = %d, ypos = %d\n",xpos,ypos);
    ret = 1;
    break;
  default:
    break;
  }
  if (!ret) //  all other events to be handled by the parent class
    ret = Fl_Select_Browser::handle(ev);
  return ret;
}
RRBrowser * browser;
RRBrowser * brws2;
//  -------------------------------------
Fl_Button *top,
    *bottom,
    *middle,
    *visible,
    *swap,
    *sort;
Fl_Choice       *btype;
Fl_Int_Input  *field;

void b_cb(Fl_Widget* o, void*) {
  printf("callback, selection = %d, event_clicks = %d\n",
   ((Fl_Browser*)o)->value(), Fl::event_clicks());
}

void show_cb(Fl_Widget *o, void *) {
  int line = atoi(field->value());

  if (!line) {
    fl_alert("Please enter a number in the text field\n"
             "before clicking on the buttons.");
    return;
  }

  if (o == top)
    browser->topline(line);
  else if (o == bottom)
    browser->bottomline(line);
  else if (o == middle)
    browser->middleline(line);
  else
    browser->make_visible(line);
}

void swap_cb(Fl_Widget *, void *) {
  int a = -1, b = -1;
  for ( int t=0; t<browser->size(); t++ ) { // find two selected items
    if ( browser->selected(t) ) {
      if ( a < 0 )
  { a = t; }
      else
  { b = t; break; }
    }
  }
  browser->swap(a, b);        // swap them
}

void sort_cb(Fl_Widget *, void *) {
  browser->sort(FL_SORT_ASCENDING);
}

void btype_cb(Fl_Widget *, void *) {
  for ( int t=1; t<=browser->size(); t++ ) browser->select(t,0);
  browser->select(1,0);   // leave focus box on first line
       if ( strcmp(btype->text(),"Normal")==0) browser->type(FL_NORMAL_BROWSER);
  else if ( strcmp(btype->text(),"Select")==0) browser->type(FL_SELECT_BROWSER);
  else if ( strcmp(btype->text(),"Hold"  )==0) browser->type(FL_HOLD_BROWSER);
  else if ( strcmp(btype->text(),"Multi" )==0) browser->type(FL_MULTI_BROWSER);
  browser->redraw();
}

int main(int argc, char **argv) {
  int i;

  setvbuf(stdout,NULL,_IONBF,0); // set stdout unbuffered for Windows/MinGW

  if (!Fl::args(argc,argv,i)) Fl::fatal(Fl::help);
  const char* fname = (i < argc) ? argv[i] : "rrbrowser.cxx";
  Fl::dnd_text_ops(1);		//	enable text ops
  Fl_Double_Window window(560,400,fname);
  browser = new RRBrowser(0,0,560,175,0);
  // browser->type(FL_MULTI_BROWSER);
  //browser->type(FL_HOLD_BROWSER);
  //browser->color(42);
  browser->callback(b_cb);
  // browser->scrollbar_right();
  //browser->has_scrollbar(Fl_Browser::BOTH_ALWAYS);
  if (!browser->load(fname)) {
    int done = 0;
#ifdef _MSC_VER
    // if 'browser' was started from the VisualC environment in Win32,
    // the current directory is set to the environment itself,
    // so we need to correct the browser file path
    if ( i == argc )
    {
      fname = "../../test/browser.cxx";
      done = browser->load(fname);
    }
#elif defined(__APPLE__)
    if ( i == argc )
    {
      char buf[2048];
      strcpy(buf, argv[0]);
      char *slash = strrchr(buf, '/');
      if (slash)
#if defined(USING_XCODE)
        strcpy(slash, "/../Resources/browser.cxx");
#else
  strcpy(slash, "/../../../browser.cxx");
#endif
      done = browser->load(buf);
    }
#endif
    if ( !done )
    {
      fl_message("Can't load %s, %s\n", fname, strerror(errno));
      exit(1);
    }
  }

  //  ---- the receive browser -------------
  brws2 = new RRBrowser(0,175,560,175,0);
  // brws2->type(FL_MULTI_BROWSER);
  brws2->callback(b_cb);
  brws2->add("AAAAA");
  brws2->add("BBBBB");
  brws2->add("CCCCC");
  brws2->add("DDDDD");
  brws2->add("EEEEE");
  brws2->add("FFFFF");
  //  -------------------

  browser->position(0);

  field = new Fl_Int_Input(55, 350, 505, 25, "Line #:");
  field->callback(show_cb);

  top = new Fl_Button(0, 375, 80, 25, "Top");
  top->callback(show_cb);

  bottom = new Fl_Button(80, 375, 80, 25, "Bottom");
  bottom->callback(show_cb);

  middle = new Fl_Button(160, 375, 80, 25, "Middle");
  middle->callback(show_cb);

  visible = new Fl_Button(240, 375, 80, 25, "Make Vis.");
  visible->callback(show_cb);

  swap = new Fl_Button(320, 375, 80, 25, "Swap");
  swap->callback(swap_cb);
  swap->tooltip("Swaps two selected lines\n(Use CTRL-click to select two lines)");

  sort = new Fl_Button(400, 375, 80, 25, "Sort");
  sort->callback(sort_cb);

  btype = new Fl_Choice(480, 375, 80, 25);
  btype->add("Normal");
  btype->add("Select");
  btype->add("Hold");
  btype->add("Multi");
  btype->callback(btype_cb);
  btype->value(3);
  btype->tooltip("Changes the browser type()");

  window.resizable(window);
  window.show(argc,argv);
  return Fl::run();
}

//
// End of "$Id: browser.cxx 9847 2013-03-23 21:32:34Z greg.ercolano $".
//
